package main

import (
	"context"
	"fmt"
	"io"
	"log"
	"os"

	"github.com/fgm/izidic"
	"gopkg.in/yaml.v2"

	"code.osinet.fr/fgm/sqs_demo/services"
	"code.osinet.fr/fgm/sqs_demo/services/redriver"
)

func main() {
	os.Exit(int(main2(os.Stdout, os.Args[0], os.Args[1:])))
}

func main2(w io.Writer, name string, args []string) (exitCode byte) {
	ctx := context.Background()
	dic := Resolve(w, name, args)
	r := dic.MustService(services.SvcRedriver).(redriver.Redriver)
	qURLs, err := r.ListQueues(ctx, "test")
	if err != nil {
		log.Printf("failed listing queues: %v", err)
		return 1
	}
	qMap := map[string]string{}
	for _, qURL := range qURLs {
		name, err := redriver.NameFromURL(qURL)
		if err != nil {
			log.Println(err)
			return 2
		}
		dlqURL, err := r.GetDLQ(ctx, name)
		if err != nil {
			log.Println(err)
			return 3
		}
		qMap[qURL] = dlqURL
	}
	bs, _ := yaml.Marshal(qMap)
	fmt.Fprintf(w, "%s\n", bs)
	log.Printf("exiting cleanly")
	return 0
}

func Resolve(w io.Writer, name string, args []string) *izidic.Container {
	dic := izidic.New()
	dic.Store(services.PName, name)
	dic.Store(services.PArgs, args)
	dic.Store(services.PWriter, w)

	dic.Register(services.SvcClient, services.SQSClientService)
	dic.Register(services.SvcFlags, services.FlagsService)
	dic.Register(services.SvcLister, services.ListerService)
	dic.Register(services.SvcLogger, services.LoggerService)
	dic.Register(services.SvcProducer, services.ProducerService)
	dic.Register(services.SvcRedriver, redriver.RedriverService)

	dic.MustService(services.SvcFlags) // Store generated params before freeze.
	dic.Freeze()
	return dic
}
